unit Listchge;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, StdCtrls,
     Dialogs, BrowseDr;

type
  tDirAttrib = record
                 Name     : string;
                 Recurse  : boolean;
                 Masks    : TStrings;
                 AllMasks : TStrings;
               end;
  TChooseDlg = class(TForm)
    SrcList: TListBox;
    DstList: TListBox;
    SrcLabel: TLabel;
    DstLabel: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    NewExtName: TEdit;
    DirName: TEdit;
    BrowseDirectoryDlg: TBrowseDirectoryDlg;
    CheckBoxRecurse: TCheckBox;
    OKBtn: TButton;
    CancelBtn: TButton;
    HelpBtn: TButton;
    IncludeBtn: TButton;
    IncAllBtn: TButton;
    ExcludeBtn: TButton;
    ExAllBtn: TButton;
    ButtonBrownse: TButton;
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExcAllBtnClick(Sender: TObject);
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
    procedure SetItem(List: TListBox; Index: Integer);
    function GetFirstSelection(List: TCustomListBox): Integer;
    procedure SetButtons;
    procedure NewExtNameKeyPress(Sender: TObject; var Key: Char);
    procedure SpeedButtonBrownseClick(Sender: TObject);
    procedure DirNameChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

function ModifPath (var Attrib : tDirAttrib; EditPath : boolean) : boolean;

implementation

{$R *.DFM}

function ModifPath (var Attrib : tDirAttrib; EditPath : boolean) : boolean;
var i, j, k : integer;
begin
  result := false;
  if (Attrib.Masks <> nil) and (Attrib.AllMasks <> nil) then
  for I := 0 to Attrib.Masks.Count - 1 do
  begin
    K := 0;
    for J := 0 to Attrib.AllMasks.Count - 1 do
      if Attrib.Masks.Strings [I] = Attrib.AllMasks.Strings [J - K] then
      begin
        Attrib.AllMasks.Delete (J);
        Inc (K);
      End;
  end;

  with TChooseDlg.Create (Application) do
  begin
    try
      Left := ((Application.MainForm.Width  - Width ) DIV 2) + Application.MainForm.Left;
      Top  := ((Application.MainForm.Height - Height) DIV 2) + Application.MainForm.Top;
      if Left < 0 then Left := 0;
      if Left + Width > Screen.Width then Left := Screen.Width - Width;
      if Top < 0 then Top := 0;
      if Top + Height > Screen.Height then Top := Screen.Height - Height;

      if Assigned (Attrib.AllMasks)
        then SrcList.Items.Assign (Attrib.AllMasks);
      if Assigned (Attrib.Masks)
        then DstList.Items.Assign (Attrib.Masks);
      with DirName do
      begin
        Text := Attrib.Name;
        Enabled := EditPath;
        if not Enabled
          then Font.Color := clGrayText
          else Font.Color := clWindowText;
      end;
      ButtonBrownse.Enabled := EditPath;
      SetButtons;
      DirNameChange(nil);
      CheckBoxRecurse.Checked := Attrib.Recurse;
      if ShowModal = mrOk then
      begin
        Attrib.Masks.Assign (DstList.Items);
        Attrib.Name    := DirName.Text;
        Attrib.Recurse := CheckBoxRecurse.Checked;
        result         := True;
      end;
    finally
      Free;
    end;
  end;
end;

procedure TChooseDlg.IncludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(SrcList);
  MoveSelected(SrcList, DstList.Items);
  SetItem(SrcList, Index);
end;

procedure TChooseDlg.ExcludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(DstList);
  MoveSelected(DstList, SrcList.Items);
  SetItem(DstList, Index);
end;

procedure TChooseDlg.IncAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to SrcList.Items.Count - 1 do
    DstList.Items.AddObject(SrcList.Items[I], 
      SrcList.Items.Objects[I]);
  SrcList.Items.Clear;
  SetItem(SrcList, 0);
end;

procedure TChooseDlg.ExcAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to DstList.Items.Count - 1 do
    SrcList.Items.AddObject(DstList.Items[I], DstList.Items.Objects[I]);
  DstList.Items.Clear;
  SetItem(DstList, 0);
end;

procedure TChooseDlg.MoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
    begin
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
      List.Items.Delete(I);
    end;
end;

procedure TChooseDlg.SetButtons;
var
  SrcEmpty, DstEmpty: Boolean;
begin
  SrcEmpty := SrcList.Items.Count = 0;
  DstEmpty := DstList.Items.Count = 0;
  IncludeBtn.Enabled := not SrcEmpty;
  IncAllBtn.Enabled := not SrcEmpty;
  ExcludeBtn.Enabled := not DstEmpty;
  ExAllBtn.Enabled := not DstEmpty;
end;

function TChooseDlg.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

procedure TChooseDlg.SetItem(List: TListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do
  begin
    SetFocus;
    MaxIndex := List.Items.Count - 1;
    if Index = LB_ERR then Index := 0
    else if Index > MaxIndex then Index := MaxIndex;
    Selected[Index] := True;
  end;
  SetButtons;
end;

procedure TChooseDlg.NewExtNameKeyPress(Sender: TObject; var Key: Char);
begin
  IF (Key = #13) and (NewExtName.Text <> '')
  then begin
         DstList.Items.Add (NewExtName.Text);
         NewExtName.Text := '';
         Key := #0;
         SetButtons;
       end;
end;

procedure TChooseDlg.SpeedButtonBrownseClick(Sender: TObject);
begin
  if BrowseDirectoryDlg.Execute then
  begin
    DirName.Text := BrowseDirectoryDlg.Selection;
  end;
end;

procedure TChooseDlg.DirNameChange(Sender: TObject);
begin
  OKBtn.Enabled := DirName.Text <> '';
end;

end.
