unit Main;

interface

uses
  Windows, Messages, Classes, Graphics, SysUtils, Forms, Dialogs, SaveOpt,
  XDelUtils, AboutDlg, uLogiciel, AppUpdate, StdCtrls, ComCtrls, Controls,
  ExtCtrls;

const
  cclAlert   = clRed;
  cclOk      = clGreen;
  cclWarning = clYellow;
  cclText    = clSilver;

type
  TDeleteForm = class(TForm)
    DlgPanel: TPanel;
    TextLabel: TLabel;
    ButtonStartStop: TButton;
    OptionsButton: TButton;
    HelpButton: TButton;
    ButtonQuit: TButton;
    ImageAnimate: TAnimate;
    Bevel1: TBevel;
    DetailMemo: TRichEdit;
    MyAboutDlg: TAboutDlg;
    ProgressBar: TProgressBar;
    UpdateButton: TButton;
    Update: TAppUpdate;
    procedure OptionsActionExecute(Sender: TObject);
    procedure AboutActionExecute(Sender: TObject);
    procedure ByeActionExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure StartStopSearchActionExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ActionUpdateClick(Sender: TObject);
  private
    { Dclarations prives }
    FShowDetail : boolean;
    FSearch     : boolean;
    procedure SetShowDetail (Value : boolean);
    procedure SetSearch     (Value : boolean);
  public
    { Dclarations publiques }
    // Screen
    function PercentProc (Percent : Integer; Messages : string) : boolean;
    procedure UpdateScreen (Msg : string; AddMemo : boolean);
    // Recherche
    procedure LanceExecution;
    procedure FinishExecution (Sender: TObject);
    procedure StopExecution;
    // Memo
    procedure AddLineToMemo(const Text: string;const cColor: TColor = cclText);
    procedure ClearMemo;
    // Proprit
    property ShowDetail : boolean read FShowDetail write SetShowDetail;
    property Recherche  : boolean read FSearch     write SetSearch;
  end;

var
  DeleteForm: TDeleteForm;

implementation

{$R *.DFM}

uses OptDlg, uDiskUtils, VersFile, ThreadSearch;

resourcestring
  Commentaire     = 'Permet le nettoyage des disques';
  StartFirtSearch = 'Afin de faire une premire recherche, il vous faut ' +
                    'configurer le logiciel.';
  CopyrightLn     = '%s v%d.%d %s';

  EmptyTrash      = 'Vidage de la corbeille ...';
  EmptyTemp       = 'Nettoyage du dossier temporaire ...';

  PreparePerso    = 'Prparation de la recherche des fichiers ...';
  SearchFile      = 'Recherche des fichiers ...';
  DeleteFiles     = 'Suppression des fichiers ...';
  FreePerso       = 'Arrt de la recherche ...';

  StopMsgOct      = 'Total d''octet(s) rcupr(s) : %s.';
  StopMsgTime     = 'Total de temps utilis %s.';

  ButtonStop      = '&Annuler';
  ButtonGo        = '&Dmarrer';

// Recherche

procedure TDeleteForm.FinishExecution (Sender: TObject);
begin // Fin de la recherche
  // Mais  jour le fichier si necessaire
  if Options.ActiveLog then UpdateLog (DetailMemo.Lines);
  // Prcise le statut : la recherhe est termin
  Recherche := false;
  // Ferme si necessaire
  if StopOnFinish then Close;
end;

procedure TDeleteForm.LanceExecution;
begin // Dmarage de la recherhe
  // Si on est au premier dmarrage
  if Options.FirstStart then
  begin
    MessageDlg(StartFirtSearch, mtInformation, [mbOk], 0);
    OptionsActionExecute(nil);
  end;
  ClearMemo;
  ProgressBar.Position := 0;
  Recherche            := true;
  ThSearch             := TSearchThread.Create (true);
  ThSearch.PercentProc := PercentProc;
  ThSearch.Resume;
end;

procedure TDeleteForm.StopExecution;
begin
  ThSearch.Terminate;
end;

procedure TDeleteForm.AddLineToMemo(const Text: string; const cColor: TColor = cclText);
begin
  ActiveControl := DetailMemo;
  DetailMemo.SelAttributes.Color := cColor;
  DetailMemo.Lines.Add(Text);
  DetailMemo.SelAttributes       := DetailMemo.DefAttributes;
end;

procedure TDeleteForm.ClearMemo;
var CopyR : string;
    Vers  : tVersionFile;
begin
  DetailMemo.Clear;
  Vers  := GetVersion;
  CopyR := Format (CopyrightLn, [GetRessourceFile (Application.ExeName, viProducteName),
                                 Vers.Majeure, Vers.Mineure,
                                 GetRessourceFile (Application.ExeName, viCopyrigth)]);
  AddLineToMemo (CopyR);
end;

procedure TDeleteForm.UpdateScreen (Msg : string; AddMemo : boolean);
var text, Line : string;
begin
  case cXDelStatus of
    xdsExec  : begin
                 case cXDelSearch of
                   xdsEmptyTrash     : begin
                                         Text := EmptyTrash;
                                         ImageAnimate.CommonAVI := aviEmptyRecycle;
                                       end;
                   xdsEmptyTemp      : begin
                                         Text := EmptyTemp;
                                         ImageAnimate.CommonAVI := aviEmptyRecycle;
                                       end;
                   xdsPreparePerso   : begin
                                         Text := PreparePerso;
                                         ImageAnimate.CommonAVI := aviNone;
                                       end;
                   xdsSearchFile     : begin
                                         Text := SearchFile;
                                         DeleteForm.ImageAnimate.CommonAVI := aviFindFile;
                                       end;
                   xdsDeleteFile     : begin
                                         Text := DeleteFiles;
                                         if Options.SendFileToTrash
                                           then ImageAnimate.CommonAVI := aviRecycleFile
                                           else ImageAnimate.CommonAVI := aviDeleteFile;
                                       end;
                   xdsFreePerso      : begin
                                         Text := FreePerso;
                                         ImageAnimate.CommonAVI := aviNone;
                                       end;
                 end;
                 if ImageAnimate.CommonAVI <> aviNone
                   then ImageAnimate.Active := true;
               end;
    xdsStop  : begin
                 Text := '';
                 if TimeClean in Options.EventLog then
                 begin
                   Line := Copy (Msg, 1, Pos (';', Msg) - 1);
                   Text := Text + Format (StopMsgTime, [TimToStr (StrToInt (Line))]);
                 end;
                 if [TimeClean, FreeSpace] <= Options.EventLog then
                 Text := Text + #13#10;
                 if FreeSpace in Options.EventLog then
                 begin
                   Line := Copy (Msg, Pos (';', Msg) + 1, Length (Msg));
                   Text := Text + Format (StopMsgOct, [SizeToStr (StrToInt64(Line))]);
                 end;
                 ImageAnimate.Active := False;
               end;
  end;
  TextLabel.Caption := Text;
  if AddMemo then
  begin
    if pos (#13, Text) <> 0 then
    begin
      AddLineToMemo (Copy (Text, 1, pos (#13, Text) - 1));
      AddLineToMemo (Copy (Text, pos (#13, Text) + 2, length (Text)));
    end else
    begin
      AddLineToMemo (Text);
    end;
  end;
  Application.ProcessMessages;
end;

function TDeleteForm.PercentProc (Percent : Integer; Messages : string) : boolean;
begin
  ProgressBar.Position := Percent;
  if Messages <> '' then
  AddLineToMemo (Messages);
  result := Recherche;
end;

procedure TDeleteForm.OptionsActionExecute(Sender: TObject);
begin
  if OpenOptionsDlg (Options) then
  begin
    Options.SaveToReg;
    if Options.Clr then Options.ClearRegistry;
    ShowDetail := Options.ShowDetail;
  end;
end;

procedure TDeleteForm.SetSearch (Value : boolean);
begin
  if Value <> FSearch then
  begin
    FSearch := Value;
    if Value then
    begin
      ButtonStartStop.Caption := ButtonStop;
      UpdateButton.Enabled    := false;
      OptionsButton.Enabled   := false;
      ButtonQuit.Enabled      := false;
      ImageAnimate.Visible    := true;
    end
    else
    begin
      ButtonStartStop.Caption := ButtonGo;
      OptionsButton.Enabled   := true;
      ButtonQuit.Enabled      := true;
      ButtonStartStop.Enabled := true;
      UpdateButton.Enabled    := true;
      ImageAnimate.Visible    := false;
    end;
  end;
end;

procedure TDeleteForm.SetShowDetail (Value : boolean);
begin
  if Value <> FShowDetail then
  begin
    FShowDetail := Value;
  end;
  if not Value then
  begin
    ClientHeight := DlgPanel.Height;
  end
  else
  begin
    ClientHeight := 2 * DlgPanel.Height;
  end;
end;

procedure TDeleteForm.AboutActionExecute(Sender: TObject);
begin
  MyAboutDlg.Comments := Commentaire;
  MyAboutDlg.Execute;
end;

procedure TDeleteForm.ByeActionExecute(Sender: TObject);
begin
  Close;
end;

procedure TDeleteForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not ((Left = Options.Position.X) and (Top = Options.Position.Y)) then
  begin
    Options.Position := Point (Left, Top);
    Options.SaveToReg;
  end;
  if Recherche
    then Action := caNone;
end;

procedure TDeleteForm.FormCreate(Sender: TObject);
begin
  ShowDetail           := Options.ShowDetail;
  UpdateButton.Enabled := Update.isEnabled;
  if (Options.Position.X <> 0) and (Options.Position.Y <> 0) then
  begin
    Left := Options.Position.X;
    Top  := Options.Position.Y;
  end;
  ClearMemo;
end;

procedure TDeleteForm.StartStopSearchActionExecute(Sender: TObject);
begin
  if not Recherche then
    LanceExecution
  else
  begin
    ButtonStartStop.Enabled := false;
    StopExecution;
  end;
end;

procedure TDeleteForm.FormShow(Sender: TObject);
begin
  if StartOnShow then LanceExecution;
end;

procedure TDeleteForm.ActionUpdateClick(Sender: TObject);
begin
  Update.Execute;
end;

end.
