unit OptDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ImgList, BrowseDr, ComCtrls, SaveOpt, SearchFile;

type
  TOptionsForm = class(TForm)
    OptionsPageControls: TPageControl;
    TabSheetAffichage: TTabSheet;
    TabSheetNettoyage: TTabSheet;
    TabSheetLog: TTabSheet;
    TabSheetSearch: TTabSheet;
    GroupBoxAffichage: TGroupBox;
    GroupBoxNettoyage: TGroupBox;
    GroupBoxLog: TGroupBox;
    GroupBoxSearch: TGroupBox;
    CheckBoxDetail: TCheckBox;
    CheckBoxCorbeille: TCheckBox;
    CheckBoxConfirm: TCheckBox;
    CheckBoxNodelete: TCheckBox;
    Bevel1: TBevel;
    CheckBoxActiveLog: TCheckBox;
    Label1: TLabel;
    LogStaticText: TStaticText;
    ButtonParcoure: TButton;
    CheckBoxFree: TCheckBox;
    CheckBoxTime: TCheckBox;
    Label2: TLabel;
    ButtonDeleteLog: TButton;
    ButtonShow: TButton;
    CheckBoxEmpty: TCheckBox;
    CheckBoxTemp: TCheckBox;
    CheckBoxPerso: TCheckBox;
    Bevel2: TBevel;
    DirectoryView: TTreeView;
    ButtonNew: TButton;
    ButtonEdit: TButton;
    ButtonDelete: TButton;
    BrowseDirectoryDlg: TBrowseDirectoryDlg;
    DriveImageList: TImageList;
    LabelRecurse: TLabel;
    LabelTaille: TLabel;
    BitBtnHelp: TButton;
    ButtonCancel: TButton;
    ButtonOk: TButton;
    CheckBoxEndFile: TCheckBox;
    ButtonDefault: TButton;
    procedure ButtonParcoureClick(Sender: TObject);
    procedure ButtonDeleteLogClick(Sender: TObject);
    procedure ButtonShowClick(Sender: TObject);
    procedure DirectoryViewChange(Sender: TObject; Node: TTreeNode);
    procedure ButtonNewEditClick(Sender: TObject);
    procedure ButtonDeleteClick(Sender: TObject);
  private
    { Dclarations prives }
    procedure CaseCheck;
    procedure UpdateOptions;
    procedure UpdateTreeView (ExpandNode : TPersistent);

    procedure UpdateTreeButton;
    procedure UpdateLogButton;

    function  GetLogFileName : string;
    function  LogNumChar     : integer;
    function  SizeOfLog      : Int64;

    procedure PlaceIcone (Value : TTreeNode; IsPath : boolean);
    function  SearchName (Value : TTreeNode) : string;

    function GetAllMasks : TStrings;
  public
    { Dclarations publiques }
    OptionsList : tSveOptions;
    property LogFileName : string read GetLogFileName;
  end;

function OpenOptionsDlg (var Opt : tSveOptions) : boolean;

resourcestring
  cDrive      = '(Drive)';
  cSizeLog    = 'Taille du fichier : %s';

  NotepadExec = 'notepad.exe';

implementation

{$R *.DFM}

uses Main, uDiskUtils, ShellApi, ListChge, XDelUtils;

function OpenOptionsDlg (var Opt : tSveOptions) : boolean;
var return : TModalResult;
begin
  result := false;
  with TOptionsForm.Create (Application) do
  begin
    try
      OptionsList := TSveOptions.Create;
      try
        Left := ((Application.MainForm.Width - Width) DIV 2) + Application.MainForm.Left;
        Top := ((Application.MainForm.Height - Height) DIV 2) + Application.MainForm.Top;
        if Left < 0 then Left := 0;
        if Left + Width > Screen.Width then left := Screen.Width - Width;
        if Top < 0 then Top := 0;
        if Top + Height > Screen.Height then Top := Screen.Height - Height;

        OptionsList.Assign (Opt);
        CaseCheck;
        UpdateTreeView (OptionsList.PathPerso);
        UpdateLogButton;
        return := ShowModal;
        if (return = mrOk) or (return = mrIgnore) then
        begin
          if return = mrIgnore then OptionsList.Clr := True;
          UpdateOptions;
          Opt.Assign (OptionsList);
          result := true;
        end;
      finally
        OptionsList.Free;
      end;
    finally
      Free;
    end;
  end;
end;

// Gestion du fichier Log

function TOptionsForm.SizeOfLog : Int64;
var HandleFile : integer;
begin
  result     := 0;
  HandleFile := FileOpen (LogFileName, fmOpenRead);
  if not (HandleFile > 0) then Exit;
  try
    FileSeek (HandleFile, 0, 0);
    result := FileSeek (HandleFile, 0, 2);
  finally
    FileClose (HandleFile);
  end;
end;

function TOptionsForm.GetLogFileName : string;
begin
  result := NormalDir (OptionsList.PathOfLog) + cDefaultLogName;
end;

function TOptionsForm.LogNumChar     : integer;
begin
  result := round (LogStaticText.Width div Canvas.TextWidth ('A'));
end;

procedure TOptionsForm.UpdateLogButton;
var Enable : boolean;
begin
  LabelTaille    .Caption := Format (cSizeLog, [SizeToStr (SizeOfLog)]);
  Enable                  := FileExists (LogFileName);
  ButtonShow     .Enabled := Enable;
  ButtonDeleteLog.Enabled := Enable;
end;

procedure TOptionsForm.ButtonParcoureClick(Sender: TObject);
begin
  if BrowseDirectoryDlg.Execute then
  begin
    OptionsList.PathOfLog := BrowseDirectoryDlg.Selection;
    LogStaticText.Caption := MinimizeFileName (LogFileName, LogNumChar);
  end;
end;

procedure TOptionsForm.ButtonDeleteLogClick(Sender: TObject);
begin
  if FileExists (LogFileName)
    then Sysutils.DeleteFile (LogFileName);
  UpdateLogButton;
end;

procedure TOptionsForm.ButtonShowClick(Sender: TObject);
begin
  ShellExecute (HInstance, nil, PChar (NotePadExec), PChar (LogFileName), nil, SW_SHOW);
  UpdateLogButton;
end;

// Gestion des options

procedure TOptionsForm.UpdateOptions;
begin
  // Page Affichage
  OptionsList.ShowDetail      := CheckBoxDetail   .Checked;
  // Page de nettoyage
  OptionsList.SendFileToTrash := CheckBoxCorbeille.Checked;
  OptionsList.DeleteConfirm   := CheckBoxConfirm  .Checked;
  OptionsList.NoDelete        := CheckBoxNodelete .Checked;
  // Page "Log"
  OptionsList.ActiveLog       := CheckBoxActiveLog.Checked;
  OptionsList.WrEndFile       := CheckBoxEndFile.Checked;
  OptionsList.EventLog        := [];
  if CheckBoxFree.Checked   then
  OptionsList.EventLog        := Options.EventLog + [FreeSpace];
  if CheckBoxTime.Checked   then
  OptionsList.EventLog        := Options.EventLog + [TimeClean];
  // Page Search
  OptionsList.CleanTrash      := CheckBoxEmpty    .Checked;
  OptionsList.CleanTemp       := CheckBoxTemp     .Checked;
  OptionsList.Personalize     := CheckBoxPerso    .Checked;
end;

procedure TOptionsForm.CaseCheck;
begin
  // Page Affichage
  CheckBoxDetail    .Checked := OptionsList.ShowDetail;
  // Page de nettoyage
  CheckBoxCorbeille.Checked  := OptionsList.SendFileToTrash;
  CheckBoxConfirm  .Checked  := OptionsList.DeleteConfirm;
  CheckBoxNodelete .Checked  := OptionsList.NoDelete;
  // Page "Log"
  CheckBoxActiveLog.Checked  := OptionsList.ActiveLog;
  CheckBoxEndFile.Checked    := OptionsList.WrEndFile;
  LogStaticText    .Caption  := MinimizeFileName (LogFileName, LogNumChar);
  CheckBoxFree     .Checked  := FreeSpace   in OptionsList.EventLog;
  CheckBoxTime     .Checked  := TimeClean   in OptionsList.EventLog;
  // Page Search
  CheckBoxEmpty    .Checked  := OptionsList.CleanTrash;
  CheckBoxTemp     .Checked  := OptionsList.CleanTemp;
  CheckBoxPerso    .Checked  := OptionsList.Personalize;
end;

// Gestion de l'arbre

procedure TOptionsForm.PlaceIcone (Value : TTreeNode; IsPath : boolean);
var Image : integer;
begin
  case Value.Level of
    0  : begin
           Value.Text          := cDrive;
           Value.ImageIndex    := 0;
           Value.SelectedIndex := 0;
         end;
    1  : begin
           case DriveType (Value.Text [1]) of
             Removable  : Image := 1;
             Fixed      : Image := 2;
             CDROM      : Image := 3;
             Remote     : Image := 4;
             else Image := -1;
           end;
           Value.ImageIndex    := Image;
           Value.SelectedIndex := Image;
         end;
    else begin
           if IsPath then
           begin
             Value.ImageIndex    := 5;
             Value.SelectedIndex := 6;
           end else
           begin
             Value.ImageIndex    := 7;
             Value.SelectedIndex := 7;
           end;
         end;
  end;
end;

procedure TOptionsForm.UpdateTreeView (ExpandNode : TPersistent);
var I, ExpandIndex : integer;
    Root, Drive    : TTreeNode;

  procedure Install (XTree : tXDelTree; Node : tTreeNode);
  var Noeud : tTreeNode;
      I     : integer;
  begin
    if XTree.Recurse
      then Node.StateIndex := 0
      else Node.StateIndex := 1;
    if (ExpandNode is TXDelTree) then
    begin
      if (ExpandNode as tXDelTree) = XTree then
      ExpandIndex := Node.AbsoluteIndex;
    end;
    For I := 0 to XTree.Masks.Count - 1 do
    begin
      Noeud := DirectoryView.Items.AddChild (Node, XTree.Masks.Strings [I]);
      PlaceIcone (Noeud, false);
    end;
    for I := 0 to XTree.Count - 1 do
    begin
      Noeud := DirectoryView.Items.AddChild (Node, XTree.Items [I].Text);
      PlaceIcone (Noeud, true);
      Install (XTree.Items [I], Noeud);
    end;
  end;

begin
  DirectoryView.Items.BeginUpdate;
  if ExpandNode is TXDelDrive
    then ExpandIndex := 0;
  with DirectoryView.Items do
  begin
    Clear;
    Root := Add (nil, cDrive);
    PlaceIcone (Root, false);
    For i := 0 to OptionsList.PathPerso.Count - 1 do
    begin
      if DriveType (OptionsList.PathPerso.DriveLetter [I]) <> no_root then
      begin
        Drive := AddChild (Root, OptionsList.PathPerso.DriveLetter [I] + ':\');
        PlaceIcone (Drive, false);
        Install (OptionsList.PathPerso.Items [I], Drive);
      end;
    end;
  end;
  DirectoryView.Items [ExpandIndex].MakeVisible;
  DirectoryView.Items [ExpandIndex].Expand (false);
  DirectoryView.Items [ExpandIndex].Selected := True;
  DirectoryView.Items.EndUpdate;
  UpdateTreeButton;
end;

procedure TOptionsForm.UpdateTreeButton;
var Enable : boolean;
begin
  Enable := DirectoryView.Selected <> nil;
  if Enable
    then Enable := DirectoryView.Selected.Level > 0;
  ButtonEdit  .Enabled := Enable;
  if Enable
    then Enable := DirectoryView.Selected.Level > 1;
  ButtonDelete.Enabled := Enable;
end;

function TOptionsForm.GetAllMasks : TStrings;
  procedure AddMask (Position : TXDelTree;var List : TStrings);
  var I : integer;
  begin
    for I := 0 to Position.Masks.Count - 1 do
    begin
      if List.IndexOf (Position.Masks.Strings [I]) = -1 then
      begin
        List.Add (Position.Masks.Strings [I]);
      end;
    end;
    for I := 0 to Position.Count - 1 do
    begin
      AddMask (Position.Items [I], List);
    end;
  end;
var Position : TXDelTree;
    I        : integer;
begin
  result   := TStringList.Create;
  For I := 0 to OptionsList.PathPerso.Count - 1 do
  begin
    Position := OptionsList.PathPerso.Items [I];
    AddMask (Position, Result);
  end;
end;

procedure TOptionsForm.DirectoryViewChange(Sender: TObject;
  Node: TTreeNode);
var TextLabel : string;
begin
  UpdateTreeButton;
  TextLabel := '[';
  if DirectoryView.Selected.StateIndex = 0
   then TextLabel := TextLabel + 'R';
  TextLabel := TextLabel + ']';
  LabelRecurse.Caption := TextLabel;
end;

function TOptionsForm.SearchName (Value : TTreeNode) : string;
var Position : TTreeNode;
begin
  result   := '';
  Position := Value;
  While Position.Level > 0 do
  begin
    if Position.Level > 1
      then result := Position.Text + '\' + Result
      else result := Position.Text +       Result;
    Position := Position.Parent;
  end;
end;

procedure TOptionsForm.ButtonNewEditClick(Sender: TObject);
var Attr                  : tDirAttrib;
    NewPath               : boolean;
    Position              : TXDelTree;
    Dossier,
    CreatedDir, CreateDir : string;
begin
  Attr.Masks    := tStringList.Create;
  case (Sender as TButton).Tag of
    0 : begin
          NewPath       := True;
          Attr.Recurse  := True;
        end;
    1 : begin
          NewPath       := False;
          Position      := OptionsList.PathPerso.GetTree (SearchName (DirectoryView.Selected));
          Attr.Name     := NormalDir (Position.Path);
          Attr.Recurse  := Position.Recurse;
          Attr.Masks.Assign (Position.Masks);
        end;
    else Exit;
  end;
  Attr.AllMasks := GetAllMasks;
  if ModifPath (Attr, NewPath) then
  begin
    Dossier    := NormalDir (Attr.Name);
    Position   := OptionsList.PathPerso.GetTree (Dossier);
    CreatedDir := NormalDir (Position.Path);
    CreateDir  := UpperCase (Dossier);
    Delete (CreateDir, Pos (CreatedDir, CreateDir), length (CreatedDir));
    if CreateDir <> '' then
    begin
      Position := Position.AddChild (CreateDir, Attr.Recurse, Attr.Masks);
    end else
    begin
      Position.Masks   := Attr.Masks;
      Position.Recurse := Attr.Recurse;
    end;
    UpdateTreeView (Position);
  end;
end;

procedure TOptionsForm.ButtonDeleteClick(Sender: TObject);
var
  XDTree     : tXDelTree;
  ParentTree : tPersistent;
  S1, S2     : string;
  Index      : integer;
begin
  S1 := UpperCase (NormalDir (SearchName (DirectoryView.Selected)));
  XDTree := OptionsList.PathPerso.GetTree (S1);
  S2 := UpperCase (NormalDir (XDTree.Path));
  ParentTree := XDTree.Parent;
  if S1 <> S2 then
  begin
    S1 := DirectoryView.Selected.Text;
    Index := XDTree.Masks.IndexOf (S1);
    XDTree.Masks.Delete (index);
  end else
  begin
    XDTree.DeleteSelf;
  end;
  UpdateTreeView (ParentTree);
end;

end.
