unit SaveOpt;

interface

uses Windows, Classes, Registry, uLogiciel, RegistryComp, VersFile, SearchFile, uDiskUtils;

type
  tEventLog   = set of (FreeSpace, TimeClean);
  tViewPage   = record
                  ShowDetail      : boolean;
                end;
  tCleanPage  = record
                  SendFileToTrash,
                  DeleteConfirm,
                  NoDelete        : boolean;
                end;
  tLogPage    = record
                  ActiveLog,
                  WrEndFile       : boolean;
                  PathOfLog       : string;
                  EventLog        : tEventLog;
                end;
  tSearchPage = record
                  CleanTrash,
                  CleanTemp,
                  Personalize : boolean;
                  PathPerso   : TMemoryStream;
                  XDPathPerso : tXDelDrive;
                end;
  tWindowsOpt = record
                  Position   : TPoint;
                  StartSearch, StopApp,
                  AutoStart  : boolean;
                end;
  tSveOptions = class (TBaseReg)
                private
                  { Dclarations prives }
                  FClear      : boolean;
                  FPageView   : tViewPage;
                  FCleanPage  : tCleanPage;
                  FLogPage    : tLogPage;
                  FSearchPage : tSearchPage;
                  FWindowsOpt : tWindowsOpt;
                  FRegOptions : tRegistry;
                  procedure DefaultPathPerso;
                protected
                  { Dclarations protg }
                  function Load     : boolean; override;
                  function Save     : boolean; override;
                  function UpdateReg (OldVersion   : tVersionFile) : boolean; override;
                public
                  { Dclarations publiques }
                  constructor Create;
                  destructor Destroy; override;
                  procedure AssignTo (Dest: TPersistent); override;
                  procedure SetDefault;

                  // Page Affichage
                  property ShowDetail      : boolean       read FPageView.ShowDetail       write FPageView.ShowDetail;
                  // Page Nettoyage
                  property SendFileToTrash : boolean       read FCleanPage.SendFileToTrash write FCleanPage.SendFileToTrash;
                  property DeleteConfirm   : boolean       read FCleanPage.DeleteConfirm   write FCleanPage.DeleteConfirm;
                  property NoDelete        : boolean       read FCleanPage.NoDelete        write FCleanPage.NoDelete;
                  // Page de "Log"
                  property ActiveLog       : boolean       read FLogPage.ActiveLog         write FLogPage.ActiveLog;
                  property WrEndFile       : boolean       read FLogPage.WrEndFile         write FLogPage.WrEndFile;
                  property PathOfLog       : string        read FLogPage.PathOfLog         write FLogPage.PathOfLog;
                  property EventLog        : tEventLog     read FLogPage.EventLog          write FLogPage.EventLog;
                  // Page de recherche
                  property CleanTrash      : boolean       read FSearchPage.CleanTrash     write FSearchPage.CleanTrash;
                  property CleanTemp       : boolean       read FSearchPage.CleanTemp      write FSearchPage.CleanTemp;
                  property Personalize     : boolean       read FSearchPage.Personalize    write FSearchPage.Personalize;
                  property PathPerso       : tXDelDrive    read FSearchPage.XDPathPerso    write FSearchPage.XDPathPerso;
                  // Aute options de X-Del
                  property Position        : TPoint        read FWindowsOpt.Position       write FWindowsOpt.Position;
                  property AutoStart       : boolean       read FWindowsOpt.AutoStart      write FWindowsOpt.AutoStart;
                  property StartSearch     : boolean       read FWindowsOpt.StartSearch    write FWindowsOpt.StartSearch;
                  property StopApp         : boolean       read FWindowsOpt.StopApp        write FWindowsOpt.StopApp;
                  // Proprit non sauvegard
                  property Clr             : boolean       read FClear                     write FClear;
                end;

resourcestring
  cViewPage      = 'Options';
    cShowDetail  = 'Show Detail';
  cCleanPage     = 'Clean Options';
    cSndFl2Trash = 'Send File To Trash';
    cDlteConfirm = 'Confirm Delete';
    cNoDelete    = 'No Delete';
  cLogPage       = 'Log Options';
    cActiveLog   = 'Active';
    cWrEndFile   = 'Write after End file';
    cPathOfLog   = 'Path';
    cEventLog    = 'Event';
  cSearchPage    = 'Search Options';
    cCleanTrash  = 'Clean Trash';
    cCleanTemp   = 'Clean Temporary Directory';
    cPersonalize = 'Personalize';
    cPathPerso   = 'Personalize Tree';
  cWindowsOpt    = 'Windows Options';
    cPositionX   = 'PosX';
    cPositionY   = 'PoxY';
    cAutoStart   = 'Dmarrage automatique';
    cStartSearch = 'Start Search';
    cStopApp     = 'Stop Application';

  NotLoadReg     = 'Ne peut charger les donnes de la base de registre.';

var
  Options : tSveOptions;

function InitOptions : boolean;

function DoneOptions : boolean;

function EventLogToStr (Event : tEventLog) : string;

function StrToEventLog (Event : string) : tEventlog;

implementation

uses SysUtils, Forms, Dialogs, Main;

function InitOptions : boolean;
begin
  result := true;
  try
    Options := TSveOptions.Create;
    if not Options.LoadFromReg then
    begin
      MessageDlg (NotLoadReg, mtError, [mbOk], 0);
    end;
  except
    result := false;
  end;
end;

function DoneOptions : boolean;
begin
  result := true;
  try
    Options.free;
  except
    result := false;
  end;
end;

function EventLogToStr (Event : tEventLog) : string;
begin
  result := IntToStr (Integer (FreeSpace  in Event)) +
            IntToStr (Integer (TimeClean  in Event));
end;

function StrToEventLog (Event : string) : tEventlog;
var I      : integer;
    vEvent : tEventLog;
begin
  result := [];
  For I := 1 to 2 do
  begin
    case I of
      1 : vEvent := [FreeSpace];
      2 : vEvent := [TimeClean];
      else vEvent := [];
    end;
    if Event [I] = '1' then
    result := result + vEvent;
  end;
end;

procedure tSveOptions.AssignTo(Dest: TPersistent);
begin
  inherited AssignTo (Dest);
  (Dest as tSveOptions).FPageView.ShowDetail  := ShowDetail;

  (Dest as tSveOptions).FCleanPage.SendFileToTrash := SendFileToTrash;
  (Dest as tSveOptions).FCleanPage.DeleteConfirm   := DeleteConfirm;
  (Dest as tSveOptions).FCleanPage.NoDelete        := NoDelete;

  (Dest as tSveOptions).FLogPage.ActiveLog := ActiveLog;
  (Dest as tSveOptions).FLogPage.WrEndFile := WrEndFile;
  (Dest as tSveOptions).FLogPage.PathOfLog := PathOfLog;
  (Dest as tSveOptions).FLogPage.EventLog  := EventLog;

  (Dest as tSveOptions).FWindowsOpt.Position    := Position;
  (Dest as tSveOptions).FWindowsOpt.AutoStart   := AutoStart;
  (Dest as tSveOptions).FWindowsOpt.StartSearch := StartSearch;
  (Dest as tSveOptions).FWindowsOpt.StopApp     := StopApp;

  (Dest as tSveOptions).FSearchPage.CleanTrash  := CleanTrash;
  (Dest as tSveOptions).FSearchPage.CleanTemp   := CleanTemp;
  (Dest as tSveOptions).FSearchPage.Personalize := Personalize;
  (Dest as tSveOptions).FSearchPage.PathPerso.LoadFromStream (FSearchPage.PathPerso);
  (Dest as tSveOptions).FSearchPage.XDPathPerso.Assign (FSearchPage.XDPathPerso);

  (Dest as tSveOptions).FClear                  := FClear;
end;

procedure tSveOptions.SetDefault;
begin
  if Assigned(FSearchPage.PathPerso) then
    FSearchPage.PathPerso.Free;
  if Assigned(FSearchPage.XDPathPerso) then
    FSearchPage.XDPathPerso.Free;
  FSearchPage.PathPerso   := TMemoryStream.Create;
  FSearchPage.XDPathPerso := tXDelDrive.Create;

  // Page d'affichage
  FPageView.ShowDetail  := False;

  // Page de Nettoyage
  FCleanPage.SendFileToTrash := True;
  FCleanPage.DeleteConfirm   := True;
  FCleanPage.NoDelete        := False;

  // Page de "Log"
  FLogPage.ActiveLog         := True;
  FLogPage.WrEndFile         := True;
  FLogPage.PathOfLog         := ExtractFilePath (Application.ExeName);
  FLogPage.EventLog          := [FreeSpace];

  // Page de Recherche
  FSearchPage.CleanTrash        := True;
  FSearchPage.CleanTemp         := True;
  FSearchPage.Personalize       := True;
  FSearchPage.PathPerso.SetSize (0);

  // Options de la fentre
  FWindowsOpt.Position       := Point (0, 0);
  FWindowsOpt.AutoStart      := false;
  FWindowsOpt.StartSearch    := true;
  FWindowsOpt.StopApp        := true;

  // Proprit non sauvegard
  FClear                     := false;

  // Dfaut :
  FSearchPage.XDPathPerso.Count := GetNumDrive;
  DefaultPathPerso;
end;

constructor tSveOptions.Create;
begin
  inherited Create;
  SetDefault;
end;

destructor tSveOptions.Destroy;
begin
  FSearchPage.PathPerso.Free;
  inherited Destroy;
end;

procedure tSveOptions.DefaultPathPerso;
var i : integer;
begin
  for I := 0 to FSearchPage.XDPathPerso.Count - 1 do
  begin
    if DriveType (FSearchPage.XDPathPerso.DriveLetter [I]) = Fixed then
    with FSearchPage.XDPathPerso.Items [I] do
    begin
      Recurse := true;
      Masks.Add ('*.$$$');
      Masks.Add ('*.~??');
      Masks.Add ('*.bak');
      Masks.Add ('*.old');
      Masks.Add ('*.tmp');
    end;
  end;
end;

function tSveOptions.Load : boolean;
var Size    : Integer;
begin
  SetDefault;
  result := true;
  try
    FRegOptions := TRegistry.Create;
    // Page d'affichage
      FRegOptions.OpenKey (VersionPath + cViewPage, True);
      if FRegOptions.ValueExists (cShowDetail) then
      FPageView.ShowDetail  := FRegOptions.ReadBool (cShowDetail);
      FRegOptions.CloseKey;

    // Page de Nettoyage
      FRegOptions.OpenKey (VersionPath + cCleanPage, True);
      if FRegOptions.ValueExists (cSndFl2Trash) then
      FCleanPage.SendFileToTrash := FRegOptions.ReadBool (cSndFl2Trash);
      if FRegOptions.ValueExists (cDlteConfirm) then
      FCleanPage.DeleteConfirm   := FRegOptions.ReadBool (cDlteConfirm);
      if FRegOptions.ValueExists (cNoDelete) then
      FCleanPage.NoDelete        := FRegOptions.ReadBool (cNoDelete);
      FRegOptions.CloseKey;

    // Page de "Log"
      FRegOptions.OpenKey (VersionPath + cLogPage, True);
      if FRegOptions.ValueExists (cActiveLog) then
      FLogPage.ActiveLog         := FRegOptions.ReadBool   (cActiveLog);
      if FRegOptions.ValueExists (cWrEndFile) then
      FLogPage.WrEndFile         := FRegOptions.ReadBool (cWrEndFile);
      if FRegOptions.ValueExists (cPathOfLog) then
      FLogPage.PathOfLog         := FRegOptions.ReadString (cPathOflog);
      if FRegOptions.ValueExists (cEventLog) then
      FLogPage.EventLog          := StrToEventLog (FRegOptions.ReadString (cEventLog));
      FRegOptions.CloseKey;

    // Page de Recherche
      FRegOptions.OpenKey (VersionPath + cSearchPage, True);
      if FRegOptions.ValueExists (cCleanTrash) then
      FSearchPage.CleanTrash     := FRegOptions.ReadBool (cCleanTrash);
      if FRegOptions.ValueExists (cCleanTemp) then
      FSearchPage.CleanTemp      := FRegOptions.ReadBool (cCleanTemp);
      if FRegOptions.ValueExists (cPersonalize) then
      FSearchPage.Personalize    := FRegOptions.ReadBool (cPersonalize);
      if FRegOptions.ValueExists (cPathPerso) then
      begin
        Size := FRegOptions.GetDataSize (cPathPerso);
        if Size = -1 then Size := 0;
        FSearchPage.PathPerso.SetSize (Size);
        FRegOptions.ReadBinaryData (cPathPerso, FSearchPage.PathPerso.Memory^, Size);
        FSearchPage.XDPathPerso.LoadFromStream (FSearchPage.PathPerso);
        FSearchPage.XDPathPerso.Count := GetNumDrive;
      end else
      begin
        FSearchPage.XDPathPerso.Count := GetNumDrive;
        DefaultPathPerso;
      end;
      FRegOptions.CloseKey;

    // Options de la fentre
      FRegOptions.OpenKey (VersionPath + cWindowsOpt, True);
      if FRegOptions.ValueExists (cPositionX) then
      FWindowsOpt.Position.X     := FRegOptions.ReadInteger (cPositionX);
      if FRegOptions.ValueExists (cPositionY) then
      FWindowsOpt.Position.Y     := FRegOptions.ReadInteger (cPositionY);
      if FRegOptions.ValueExists (cAutoStart) then
      FWindowsOpt.AutoStart      := FRegOptions.ReadBool    (cAutoStart);
      if FRegOptions.ValueExists (cStartSearch) then
      FWindowsOpt.StartSearch    := FRegOptions.ReadBool    (cStartSearch);
      if FRegOptions.ValueExists (cStopApp) then
      FWindowsOpt.StopApp        := FRegOptions.ReadBool    (cStopApp);
      FRegOptions.CloseKey;
    FRegOptions.Free;
  except
    result := false;
  end;
end;

function tSveOptions.Save : boolean;
begin
  result := true;
  try
    FRegOptions := TRegistry.Create;
    // Page d'affichage
      FRegOptions.OpenKey (VersionPath + cViewPage, True);
      FRegOptions.WriteBool (cShowDetail, FPageView.ShowDetail);
      FRegOptions.CloseKey;

    // Page de Nettoyage
      FRegOptions.OpenKey (VersionPath + cCleanPage, True);
      FRegOptions.WriteBool (cSndFl2Trash, FCleanPage.SendFileToTrash);
      FRegOptions.WriteBool (cDlteConfirm, FCleanPage.DeleteConfirm);
      FRegOptions.WriteBool (cNoDelete, FCleanPage.NoDelete);
      FRegOptions.CloseKey;

    // Page de "Log"
      FRegOptions.OpenKey (VersionPath + cLogPage, True);
      FRegOptions.WriteBool   (cActiveLog, FLogPage.ActiveLog);
      FRegOptions.WriteBool   (cWrEndFile, FLogPage.WrEndFile);
      FRegOptions.WriteString (cPathOflog, FLogPage.PathOfLog);
      FRegOptions.WriteString (cEventLog, EventLogToStr (FLogPage.EventLog));
      FRegOptions.CloseKey;

    // Page de Recherche
      FRegOptions.OpenKey (VersionPath + cSearchPage, True);
      FRegOptions.WriteBool (cCleanTrash, FSearchPage.CleanTrash);
      FRegOptions.WriteBool (cCleanTemp, FSearchPage.CleanTemp);
      FRegOptions.WriteBool (cPersonalize, FSearchPage.Personalize);
      FSearchPage.XDPathPerso.SaveToStream (FSearchPage.PathPerso);
      if FSearchPage.PathPerso <> nil then
      FRegOptions.WriteBinaryData (cPathPerso, FSearchPage.PathPerso.Memory^, FSearchPage.PathPerso.Size);
      FRegOptions.CloseKey;

    // Options de la fentre
      FRegOptions.OpenKey (VersionPath + cWindowsOpt, True);
      FRegOptions.WriteInteger (cPositionX, FWindowsOpt.Position.X);
      FRegOptions.WriteInteger (cPositionY, FWindowsOpt.Position.Y);
      FRegOptions.WriteBool    (cAutoStart, FWindowsOpt.AutoStart);
      FRegOptions.WriteBool    (cStartSearch, FWindowsOpt.StartSearch);
      FRegOptions.WriteBool    (cStopApp, FWindowsOpt.StopApp);
      FRegOptions.CloseKey;
    FRegOptions.Free;
  except
    result := false;
  end;
end;

function tSveOptions.UpdateReg (OldVersion   : tVersionFile) : boolean;
var Old: tVersionFile;
begin
  result := true;
  if DecomposeVersion (OldVersion) = '0.0.0.0' then Exit;
  if CompareVersion   (OldVersion, ComposeVersion('3.0.0.0')) = 1
    then begin
           Old      := FVersion;
           FVersion := OldVersion;
           Load;
           FVersion := Old;
         end
    else result := false;
  // No implemente for this time
end;

initialization

finalization
  DoneOptions;

end.
