unit SearchFile;

interface

uses Classes, SysUtils, comctrls, uDiskUtils, DebugBox, uList;

type
  tSearchEventFile = function (Percent : integer; Path : string; SRec : tSearchRec) : boolean of object;
  tSearchEventDir  = function (Percent : integer; Path : string)                    : boolean of object;
  EXDel            = class(Exception);
  ERestart         = class(Exception);
  tXDelTree = class (tPersistent)
  private
    { dclarations prives }
    FItems   : TList;
    FText    : string;
    FRecurse : boolean;
    FMasks   : tStrings;
    FParent  : tPersistent;
    function  GetItems (Index : integer) : tXDelTree;
    procedure SetItems (Index : integer; Value : tXDelTree);
    function  GetPath                    : string;
    procedure SetText  (Value : string);
    function  GetLevel                   : integer;
    function  GetCount                   : integer;
    procedure SetMasks (Value : TStrings);
    function  GetPrimaryList             : TStrings;
    function  GetListMasks               : TStrings;
  protected
    { dclarations protges }
  public
    { dclarations publiques }
    constructor Create (Owner : tPersistent);
    destructor  Destroy; override;
    procedure AssignTo(Dest: TPersistent); override;

    function AddChild (Txt : string; Recurse : boolean; Masks : TStrings) : tXDelTree;
    function AddTree (Arbre : TXDelTree) : boolean;
    procedure Delete (Index : integer);
    procedure DeleteSelf;

    function GetIndex (S : string) : integer;
    function GetTree (S : string) : TXDelTree;

    procedure LoadFromStream (St : tstream);
    procedure SaveToStream (St : tStream);

    procedure Execute (OnFindDir : tSearchEventDir;OnFindFile : tSearchEventFile;
                       ParentPourcentage, ParentStep : real);

    property Items [index : integer] : tXDelTree read GetItems write SetItems;
  published
    { dclarations publie }
    property Text                    : string      read FText    write SetText;
    property Path                    : string      read GetPath;
    property Recurse                 : boolean     read FRecurse write FRecurse;
    property Masks                   : TStrings    read FMasks   write SetMasks;
    property Parent                  : tPersistent read FParent;
    property Level                   : Integer     read GetLevel;
    property Count                   : Integer     read GetCount;
    // Recherche
    property PrimaryList             : TStrings    read GetPrimaryList;
    property ListMasks               : TStrings    read GetListMasks;
  end;

  tXDelDrive = class (tPersistent)
  private
    { dclarations prives }
    FItems   : TList;
    FCount   : integer;
    procedure SetCount (Value : integer);
    function  GetDiskInDrive (index : integer) : boolean;
    function  GetDriveLetter (index : integer) : Char;
    function GetItems (Index : Integer) : TXDelTree;
    procedure SetItems (Index : integer; Value : tXDelTree);
  protected
    { dclarations protges }
  public
    { dclarations publiques }
    constructor Create;
    destructor Destroy; override;
    procedure AssignTo(Dest: TPersistent); override;

    function GetTree (S : string) : TXDelTree;

    procedure LoadFromFile (const FileName: string);
    procedure LoadFromStream (St : tstream);
    procedure SaveToFile (const FileName: string);
    procedure SaveToStream (St : tStream);

    procedure Execute (OnFindDir : tSearchEventDir;OnFindFile : tSearchEventFile);

    property Items         [Index : integer] : tXDelTree read GetItems write SetItems;
    property DriveLetter   [index : integer] : Char      read GetDriveLetter;
    property IsDiskInDrive [index : integer] : boolean   read GetDiskInDrive;
  published
    { dclarations publie }
    property Count                           : integer   read FCount   write SetCount;
  end;

resourcestring
  UserHalt  = 'Interruption utilisateur';
  Restart   = 'Changement : redmarrage de la recherche';

const
  PathError : string = 'Pas d''erreur';

implementation

uses Dialogs;

// TXDelDrive

procedure Error;
begin
  raise EXDel.Create(UserHalt);
end;

constructor TXDelDrive.Create;
begin
  inherited Create;
  FItems := TMemList.Create;
end;

destructor TXDelDrive.Destroy;
begin
  FItems.Free;
  inherited Destroy;
end;

procedure TXDelDrive.AssignTo(Dest: TPersistent);
var I : integer;
begin
  (Dest as tXDelDrive).Count := FCount;
  For I := 0 to FCount - 1 do
  begin
    (Dest as tXDelDrive).Items [I].Assign (Items [I]);
  end;
end;

function TXDelDrive.GetDriveLetter (index : integer) : Char;
begin
  result := Chr (Index + $41);
end;

function TXDelDrive.GetDiskInDrive (index : integer) : boolean;
begin
  result := DiskInDrive (GetDriveLetter (Index));
end;

procedure TXDelDrive.SetCount (Value : integer);
var I     : integer;
    Arbre : TXDelTree;
begin
  if Value <> FCount then
  begin
    if Value > FCount then
    begin
      for I := FCount to Value - 1 do
      begin
        Arbre      := TXDelTree.Create (Self);
        Arbre.Text := GetDriveLetter (I) + ':';
        FItems.Add (Arbre);
      end;
    end else
    begin
      for I := Value to FCount do
      begin
        FItems.Delete (FItems.Count - 1);
      end;
    end;
    FCount := Value;
  end;
end;

procedure TXDelDrive.SetItems (Index : integer; Value : tXDelTree);
begin
  FItems [index] := Value;
end;

function TXDelDrive.GetItems (Index : Integer) : TXDelTree;
begin
  result := FItems [index];
end;

function TXDelDrive.GetTree (S : string) : TXDelTree;
var Disque  : char;
    Dossier : string;
begin
  if S = '' then
  begin
    result := nil;
    exit;
  end;
  Disque := S [1];
  Dossier := Copy (NormalDir (S), Pos ('\', NormalDir (S)) + 1, length (NormalDir (S)));
  result := Items [Ord (Disque) - $41].GetTree (Dossier);
end;

procedure tXDelDrive.SaveToStream (St : tStream);
var Arbre   : tMemoryStream;
    Size, I : Integer;
begin
  St.Size := 0;
  St.WriteBuffer (Count, SizeOf (Integer));
  for I := 0 to Count - 1 do
  begin
    Arbre := tMemoryStream.Create;
    Items [I].SaveToStream (Arbre);
    Size := Arbre.Size;
    St.WriteBuffer (Size, sizeOf (Integer));
    St.WriteBuffer (Arbre.Memory^, Size);
    Arbre.Free;
  end;
end;

procedure tXDelDrive.LoadFromStream (St : tstream);
var Arbre   : tMemoryStream;
    I, Size : Integer;
begin
  if St.Size = 0 then Exit;
  St.Seek (0, 0);
  St.ReadBuffer (I, SizeOf (Integer));
  Count := I;
  for I := 0 to Count - 1 do
  begin
    St.ReadBuffer (Size, SizeOf (Integer));
    Arbre := tMemoryStream.Create;
    Arbre.SetSize (Size);
    St.ReadBuffer (Arbre.Memory^, Size);
    Items [I].LoadFromStream (Arbre);
  end;
end;

procedure TXDelDrive.LoadFromFile (const FileName: string);
var Stream : tFileStream;
begin
  Stream := tFileStream.Create (FileName, fmOpenRead);
  LoadFromStream (Stream);
  Stream.Free;
end;

procedure TXDelDrive.SaveToFile (const FileName: string);
var Stream : tFileStream;
begin
  Stream := tFileStream.Create (FileName, fmCreate);
  SaveToStream (Stream);
  Stream.Free;
end;

procedure TXDelDrive.Execute (OnFindDir : tSearchEventDir;OnFindFile : tSearchEventFile);
var I, J  : integer;
    List              : TStrings;
    Tree              : tXDelTree;
    Pourcentage, Step : real;
begin
  if not Assigned (OnFindFile) then Exit;
  Step  := 100 / FCount;
  for i := 0 to FCount - 1 do
  begin
    Pourcentage := (I * 100) / FCount;
    if Assigned(OnfindDir)
      then OnFindDir(round (Pourcentage), DriveLetter[I]);
    List := TStringList.Create;
    try
      List.Assign (Items [I].GetPrimaryList);
      if (List.Count <> 0) and (IsDiskInDrive [I]) then
        for J := 0 to List.Count - 1 do
        begin
          Tree := GetTree (List.Strings [J]);
          Tree.Execute (OnFindDir, OnFindFile, Pourcentage, Step);
        end;
    finally
      List.Free;
    end;
  end;
end;

// TXDelTree

procedure tXDelTree.Execute (OnFindDir : tSearchEventDir;OnFindFile : tSearchEventFile;
                             ParentPourcentage, ParentStep : real);
var
  SRec        : TSearchRec;
  RetVal, I   : Integer;
  Stop        : boolean;
  Pourc, Step : real;
begin
  PathError := path;
  Stop := false;
  if Assigned(OnfindDir) then
    Stop := OnFindDir(round (ParentPourcentage), Path);
  if Stop then Error;
  // Recherche des fichiers.
  for I := 0 to ListMasks.Count - 1 do
  begin
    RetVal := FindFirst(NormalDir(Path) + ListMasks.Strings[I], faAnyFile, SRec);
    try
      while RetVal = 0 do
      begin
        Stop := OnFindFile(round (ParentPourcentage), Path, SRec);
        if Stop then Error;
        RetVal := FindNext(SRec);
      end;
    finally
      FindClose(SRec);
    end;
  end;
  // Recherche des rpertoires.
  RetVal := FindFirst(NormalDir(Path) + '*.*', faDirectory, SRec);
  try
    while RetVal = 0 do
    begin
      if ((SRec.Attr and faDirectory) <> 0) and ((SRec.Name <> '.') and (SRec.Name <> '..'))
        then AddChild(SRec.Name, true, nil);
      RetVal := FindNext(SRec);
    end;
  finally
    FindClose(SRec);
  end;
  if Count <> 0 then
  begin
    // Recursivit
    Step  := (ParentStep / 100) * Count;
    for I := 0 to Count - 1
      do if DirExists(Items[I].Path) then
         begin
           Pourc := ParentPourcentage + ((I / Count) * ParentStep);
           Items[I].Execute(OnFindDir, OnFindFile, Pourc, Step);
         end;
  end;
end;

function tXDelTree.GetTree (S : string) : TXDelTree;
var Dossier : string;
    Index   : integer;
begin
  Dossier := Copy (NormalDir (S), 1, Pos ('\', NormalDir (S)) - 1);
  Index   := GetIndex (Dossier);
  if Index = -1 then
  begin
    result := Self;
  end else
  begin
    Dossier := Copy (NormalDir (S), Pos ('\', NormalDir (S)) + 1, length (NormalDir (S)));
    result := Items [Index].GetTree (Dossier);
  end;
end;

function tXDelTree.GetListMasks : TStrings;
var List : TStrings;
    Temp : TPersistent;
    Tree : TXDelTree;
begin
  List := TStringList.Create;
  List.AddStrings (Masks);
  Temp := Parent;
  While (Temp is tXDelTree) do
  begin
    Tree := (temp as tXDelTree);
    if Tree.Recurse
      then List.AddStrings (Tree.Masks);
    Temp := Tree.Parent;
  end;
  Result := List;
end;

function tXDelTree.GetPrimaryList : TStrings;
var List : TStrings;
    I    : Integer;
begin
  List := TStringList.Create;
  if Masks.Count <> 0
    then List.Add (Path)
    else for I := 0 to Count - 1
           do List.AddStrings (Items [I].PrimaryList);
  result := List;
end;

procedure tXDelTree.AssignTo(Dest: TPersistent);
var I    : integer;
    Tree : TXDelTree;
begin
  (Dest as tXDelTree).FText := FText;
  (Dest as tXDelTree).FRecurse := FRecurse;
  (Dest as tXDelTree).FMasks.Assign (FMasks);
  (Dest as tXDelTree).FItems.Clear;
  for I := 0 to Count - 1 do
  begin
    Tree := TXDelTree.Create (Dest as TXDelTree);
    Tree.Assign (Items [I]);
    (Dest as tXDelTree).AddTree (Tree);
  end;
end;

constructor tXDelTree.Create (Owner : tPersistent);
begin
  inherited Create;
  FItems   := TMemList.Create;
  FMasks   := tStringList.Create;
  FRecurse := True;
  FParent  := Owner;
end;

destructor tXDelTree.Destroy;
begin
  FItems.Free;
  FMasks.Free;
  inherited Destroy;
end;

procedure tXDelTree.SaveToStream (St : tStream);
var ItemStream,
    MasksStream : tMemoryStream;
    aPath       : ShortString;
    Rec         : boolean;
    Size, I     : Integer;
begin
  St.Size := 0;
  // Prparation
  aPath   := FText;
  Rec     := Recurse;
  MasksStream := TMemoryStream.Create;
  Masks.SaveToStream (MasksStream);
  // Enregistrement
  St.WriteBuffer (aPath, Length (Text) + 1);
  St.WriteBuffer (Rec, SizeOf (boolean));
  Size := MasksStream.Size;
  St.WriteBuffer (Size, sizeOf (Integer));
  St.WriteBuffer (MasksStream.Memory^, Size);
  MasksStream.Free;
  For I := 0 to Count - 1 do
  begin
    ItemStream := TMemoryStream.Create;
    Items [I].SaveToStream (ItemStream);
    Size := ItemStream.Size;
    St.WriteBuffer (Size, sizeOf (Integer));
    St.WriteBuffer (ItemStream.Memory^, Size);
    ItemStream.Free;
  end;
end;

procedure tXDelTree.LoadFromStream (St : tstream);
var ItemStream,
    MasksStream : tMemoryStream;
    aPath       : ShortString;
    Rec         : boolean;
    Size        : Integer;
    Ch          : Char;
    Tmp         : TXDelTree;
begin
  St.Seek (0, 0);
  // Lecture
  St.Read (Ch, SizeOf (Char));
  St.Seek (0, 0);
  St.ReadBuffer (aPath, Ord (Ch) + 1);
  St.ReadBuffer (Rec, SizeOf (Boolean));
  Text    := aPath;
  Recurse := Rec;
  St.ReadBuffer (Size, SizeOf (Integer));
  If Size <> 0 then
  begin
    MasksStream := TMemoryStream.Create;
    MasksStream.SetSize (Size);
    St.ReadBuffer (MasksStream.Memory^, Size);
    Masks.LoadFromStream (MasksStream);
    MasksStream.Free;
  end else
  begin
    Masks.Clear;
  end;
  While St.Position < St.Size do
  begin
    St.ReadBuffer (Size, SizeOf (Integer));
    ItemStream := TMemoryStream.Create;
    ItemStream.SetSize (Size);
    St.ReadBuffer (ItemStream.Memory^, Size);
    Tmp := AddChild ('', false, nil);
    Tmp.LoadFromStream (ItemStream);
    ItemStream.Free;
  end;
end;

function tXDelTree.GetCount : integer;
begin
  result := FItems.Count;
end;

function tXDelTree.GetIndex (S : string) : integer;
var I : integer;
begin
  result := -1;
  For I := 0 to Count - 1 do
  begin
    if UpperCase (Items [I].FText) = UpperCase (S) then
    begin
      result := I;
      Break;
    end;
  end;
end;

function tXDelTree.AddTree (Arbre : TXDelTree) : boolean;
begin
  result := FItems.Add (Arbre) <> -1;
end;

function tXDelTree.AddChild (Txt : string; Recurse : boolean; Masks : TStrings) : tXDelTree;
var s               : string;
    Parcoure, Arbre : tXDelTree;
begin
  Txt := NormalDir ('\' + Txt);
  System.Delete(Txt, 1, 1);
  Parcoure := self;
  repeat
    S := Copy (txt, 1, pos ('\', txt) - 1);
    if Parcoure.GetIndex (S) <> -1 then
    begin
      Parcoure := Parcoure.Items [GetIndex (S)];
    end else
    begin
      Arbre := tXDelTree.Create (Parcoure);
      Arbre.Text     := S;
      Arbre.Recurse  := Recurse;
      Parcoure.AddTree (Arbre);
      Parcoure       := Arbre;
    end;
    System.Delete (Txt, 1, pos ('\', txt));
  until Pos ('\', Txt) = 0;
  if Masks <> nil
    then Parcoure.Masks.Assign (Masks);
  result := Parcoure;
end;

procedure tXDelTree.Delete (Index : integer);
begin
  Items [index].Free;
  FItems.Delete (index);
end;

procedure tXDelTree.DeleteSelf;
var Index : integer;
begin
  Index := (FParent as tXDelTree).GetIndex (FText);
  (FParent as tXDelTree).Delete (Index);
end;

procedure tXDelTree.SetItems (Index : integer; Value : tXDelTree);
begin
  FItems [index] := Value;
end;

procedure tXDelTree.SetMasks (Value : TStrings);
begin
  if Value <> nil
    then FMasks.Assign (Value)
    else FMasks.Clear;
end;

function tXDelTree.GetItems (Index : integer) : tXDelTree;
begin
  result := FItems [index];
end;

procedure tXDelTree.SetText (Value : string);
begin
  if Pos ('\', Value) = 0 then
  begin
    FText := UpperCase (Value);
  end;
end;

function tXDelTree.GetPath : string;
begin
  result := NormalDir (FText);
  if (Parent is tXDelTree) then
  begin
    result := NormalDir ((Parent as tXDelTree).Path + FText);
  end;
end;

function tXDelTree.GetLevel : integer;
begin
  if not (Parent is tXDelTree)
    then Result := 0
    else Result := (Parent as tXDelTree).Level + 1;
end;

end.

