unit ThreadSearch;

interface

uses
  Classes, SysUtils, XDelUtils, Dialogs, uLogiciel, uList;

type
  TPercentProc  = function (Percent : Integer; Messages : string) : boolean of object;

  TSearchThread = class(TThread)
  private
    { Dclarations prives }
    FPercentProc : TPercentProc;
    FDeleteList  : TList;
    function VideCorbeille : boolean;
    function VideTempDir : boolean;
    procedure DeleteFilesList(ListFile: TList);
    procedure SynchronizeScreen;
    procedure SynchronizePercent;
    procedure WriteTextToMemo;
    function FindDir(Percent : integer; Path : string) : boolean;
    function Findfile (Percent : integer; Path : string; SRec : tSearchRec) : boolean;
  protected
    procedure Execute; override;
  public
    constructor Create(CreateSuspend: Boolean);
    property PercentProc : TPercentProc read FPercentProc write FPercentProc;
  end;

resourcestring
  ConfirmDel  = 'tes-vous sr de vouloir supprimer ' +
                'le fichier "%s" qui est "cach" ou/et ' +
                '"en lecture seul".';

  CleanTrash  = 'Nettoyage de la corbeille.';
  CleanTemp   = 'Nettoyage du dossier temporaire : %s';
  DeleteFile  = 'Delete : %s.';
  NodelFile   = 'No delete : %s.';

  EDeleteFile = 'Erreur lors de la suppression de %s.';
  EEmptyTrash = 'Erreur lors du vidage de la corbeille.';
  EEmptyTemp  = 'Erreur lors du vidage du dossier temporaire.';
  EException  = 'Erreur : %s on %s';

var
  ThSearch        : TSearchThread;

const
  cXDelStatus     : tXDelStatus = xdsStop;
  cXDelSearch     : tXDelSearch = xdsEmptyTrash;

implementation

uses Windows, Main, Forms, uSHFileOp, SaveOpt, uWindows, uDiskUtils,
     SearchFile, ListFile, Controls, uStrUtil, Graphics;

type
  TMessAlert = (None, Alert, Warning); 

var
  FreeSpaceExec, TimeExec : Int64;
  PercentConst            : Integer;
  Msgs                    : string;
  Pos, Max, CountR        : real;

const
  TextToMemo : string     = '';
  MessType   : TMessAlert = None;

{ TSearchThread }

constructor TSearchThread.Create(CreateSuspend: Boolean);
begin
  inherited Create (CreateSuspend);
  Priority        := tpNormal;
  FreeOnTerminate := True;
  OnTerminate     := DeleteForm.FinishExecution;
  FreeSpaceExec   := 0;
  Msgs            := '';
  Max             := Ord(High(tXDelSearch)) + 1;
  Pos             := 0;
  CountR          := 1 / Ord(High(tXDelSearch));
end;

function TSearchThread.FindDir(Percent : integer; Path : string) : boolean;
begin
  if Percent > PercentConst then
  PercentConst := Percent;
  Synchronize (SynchronizePercent);
  result := Terminated;
end;

function TSearchThread.Findfile(Percent : integer; Path : string; SRec : tSearchRec) : boolean;
var Tab : PSRecPlus;
begin
  if Percent > PercentConst then
  PercentConst := Percent;
  Synchronize (SynchronizePercent);
  New(Tab);
  try
    Tab.Path     := Path;
    Tab.FileName := SRec;
    Tab.Name     := NormalDir(Path) + SRec.Name;
    FDeleteList.Add (Tab);
  except
    Dispose (Tab);
  end;
  result := Terminated;
end;

procedure TSearchThread.SynchronizePercent;
var
  PosTt, PosSearch : real;
  Perc             : Integer;
begin
  PosSearch := (PercentConst / 100) * (Max - 1);
  Postt     := Pos + PosSearch;
  Perc      := round ((PosTt * 100) / Max);
  FPercentProc (Perc, Msgs);
end;

procedure TSearchThread.SynchronizeScreen;
var Percent : Integer;
begin
  case cXDelStatus of
    xdsExec  : DeleteForm.UpdateScreen ('', false);
    xdsStop  : DeleteForm.UpdateScreen (IntToStr (TimeExec) + ';' + IntToStr (FreeSpaceExec), True);
  end;
  if Assigned (FPercentProc) then
  begin
    Percent := round ((Pos * 100) / Max);
    FPercentProc (Percent, Msgs);
  end;
end;

procedure TSearchThread.WriteTextToMemo;
var ColorType : TColor;
begin
  case MessType of
    Alert   : ColorType := cclAlert;
    Warning : ColorType := cclWarning;
    else ColorType := cclText;
  end;
  if TextToMemo <> ''
    then DeleteForm.AddLineToMemo(TextToMemo, ColorType);
  MessType := None;
  Application.ProcessMessages;
end;

function TSearchThread.VideCorbeille : boolean;
begin
  try
    result := true;
    ShellEmptyRecycleBin;
    TextToMemo := CleanTrash;
    WriteTextToMemo;
  except
    TextToMemo := EEmptyTrash;
    MessType := Alert;
    WriteTextToMemo;
    result := false;
  end;
end;

function TSearchThread.VideTempDir : boolean;
var
  ShellStruct            : TShellFileOpStruct;
  SizeAdd, BeginDiskSize : Int64;
  TmpDir                 : string;
  DriveLetter            : Byte;
begin
  try
    TmpDir             := UpperCase (GetTempDir);
    DriveLetter        := Ord (TmpDir [1]) - Ord ('A') + 1;

    BeginDiskSize      := DiskFree (DriveLetter);

    ShellStruct.wFunc  := foDelete;
    ShellStruct.pFrom  := NormalDir (TmpDir) + '*.*';
    ShellStruct.fFlags := [fofNoConfirmation, fofSilent];
    result             := ShellFileOperation (ShellStruct);

    SizeAdd            := DiskFree (DriveLetter) - BeginDiskSize;
    Inc (FreeSpaceExec, SizeAdd);
    TextToMemo := Format(CleanTemp, [SizeToStr(SizeAdd)]);
    WriteTextToMemo;
  except
    TextToMemo := EEmptyTemp;
    MessType   := Alert;
    WriteTextToMemo;
    result := false;
  end;
end;

procedure TSearchThread.DeleteFilesList(ListFile: TList);
  procedure DeleteFileByShell(FileName: TFileName; ToTrash: boolean);
  var Opt: TShellFileOpStruct;
  begin
    with Opt do
    begin
      wFunc  := foDelete;
      pFrom  := FileName;
      fFlags := [fofFilesOnly, fofNoConfirmation, fofSilent];
      if ToTrash then fFlags := fFlags + [fofAllowUndo];
      if not FileIsUse(FileName) then
      ShellFileOperation(Opt);
    end;
  end;

var I             : integer;
    Rec           : PSRecPlus;
    DeleteConfirm : boolean;
begin
  rec := nil;
  for I := 0 to ListFile.Count - 1 do
  try
    if Terminated then Break;
    Rec := ListFile.Items[I];

    DeleteConfirm := not Options.NoDelete;
    if (Rec.FileName.Attr and (faReadOnly or faHidden) > 0) and DeleteConfirm
      then DeleteConfirm := MessageDlg (Format(ConfirmDel, [Rec.Name]),
                                        mtConfirmation, [mbYes, mbNo], 0) = mrYes;
    if Rec.FileName.Attr and faSysFile > 0
      then DeleteConfirm := False;

    if DeleteConfirm and FileExists (Rec.Name) then
    begin
      FileSetAttr(Rec.Name, 0);
      DeleteFileByShell(Rec.Name, Options.SendFileToTrash);
    end;

    if not FileExists (Rec.Name) then
    begin
      TextToMemo := Format (DeleteFile, [Rec.Name]);
      FreeSpaceExec := FreeSpaceExec + Rec.FileName.Size;
    end else
    begin
      MessType   := Warning;
      TextToMemo := Format (NodelFile, [Rec.Name]);
    end;

    if debug then TextToMemo := TextToMemo + ' ' + FileAttribue (Rec.FileName);

    Synchronize(WriteTextToMemo);
  except
    if Assigned(Rec) then
    begin
      TextToMemo := Format(EDeleteFile, [Rec.Name]);
      MessType   := Alert;
      WriteTextToMemo;
    end;
  end;
end;

procedure TSearchThread.Execute;
var
  TimeStart, TimeStop : Int64;
  CanStop             : boolean;
  PathPersoSearch     : tXDelDrive;
begin
  cXDelStatus     := xdsExec;
  cXDelSearch     := Low (tXDelSearch);
  TimeStart       := GetTickCount;
  CanStop         := False;
  PathPersoSearch := nil;
  FDeleteList     := nil;
  repeat
    if Terminated
      then cXDelStatus := xdsStop;
    case cXDelStatus of
      xdsExec  : begin
                   Synchronize (SynchronizeScreen);
                   case cXDelSearch of
                     xdsEmptyTrash   : begin
                                         if Options.CleanTrash then
                                         VideCorbeille;
                                         IncReal (Pos, CountR);
                                       end;
                     xdsEmptyTemp    : begin
                                         if Options.CleanTemp  then
                                         VideTempDir;
                                         IncReal (Pos, CountR);
                                       end;
                     xdsPreparePerso : begin
                                         if Options.Personalize then
                                         begin
                                           FDeleteList     := TMemList.Create;
                                           PathPersoSearch := tXDelDrive.Create;
                                           try
                                             PathPersoSearch.Assign (Options.PathPerso);
                                           except
                                             PathPersoSearch.Free;
                                             PathPersoSearch := nil;
                                           end;
                                         end;
                                         IncReal (Pos, CountR);
                                       end;
                     xdsSearchFile   : begin
                                         if Options.Personalize then
                                         begin
                                           try
                                             PathPersoSearch.Execute (FindDir, FindFile);
                                           except
                                             on E: Exception do
                                             begin
                                               TextToMemo := Format(EException, [E.Message, PathError]);
                                               MessType   := Alert;
                                               WriteTextToMemo;
                                               if not (E is ERestart) then
                                               begin
                                                 PathPersoSearch.Free;
                                                 PathPersoSearch := nil;
                                               end;
                                             end;
                                           end;
                                         end;
                                         IncReal (Pos, Max - 1);
                                       end;
                     xdsDeleteFile   : begin
                                         if Options.Personalize then
                                         begin
                                           FDeleteList := SelectListFile(FDeleteList);
                                           DeleteFilesList(FDeleteList);
                                           IncReal (Pos, CountR);
                                         end;
                                       end;
                     xdsFreePerso    : begin
                                         if Assigned (PathPersoSearch) then
                                         PathPersoSearch.Free;
                                         if Assigned (FDeleteList) then
                                         FDeleteList.Free;
                                         IncReal (Pos, CountR);
                                       end;
                   end;
                   if cXDelSearch = High (tXDelSearch) then
                   begin
                     cXDelStatus := xdsStop;
                   end else
                   begin
                     cXDelSearch := Succ (cXDelSearch);
                   end;
                 end;
      xdsStop  : begin
                   TimeStop  := GetTickCount;
                   TimeExec  := TimeStop - TimeStart;
                   Synchronize (SynchronizeScreen);
                   CanStop   := True;
                 end;
    end;
  until CanStop;
end;

end.
